<?php
/**
 * @file
 * Database functions for OAI Harvester module
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

/**
 * Return the list of links to the providers which support a given format
 * @param $format_id The id of format
 * @return Array of links
 */
function _oaiharvester_providers_get_by_format($format_id) {
  $sql = 'SELECT prov.provider_id, prov.name
            FROM {oaiharvester_formats_to_providers} AS ftp
            LEFT JOIN {oaiharvester_providers} AS prov ON prov.provider_id = ftp.provider_id
              WHERE ftp.format_id = %d
                ORDER BY prov.name';
  $result = db_query($sql, $format_id);
  $providers = array();
  while ($data = db_fetch_object($result)) {
    $providers[] = l(check_plain($data->name),
      'admin/xc/harvester/repository/' . check_plain($data->provider_id));
  }
  return $providers;
}

/**
 * Return the list of links to the providers which support a given set
 * @param $format_id The id of set
 * @return Array of links
 */
function _oaiharvester_providers_get_by_set($set_id) {
  $sql = 'SELECT prov.provider_id, prov.name
            FROM {oaiharvester_sets_to_providers} AS stp
            LEFT JOIN {oaiharvester_providers} AS prov ON prov.provider_id = stp.provider_id
              WHERE stp.set_id = %d
                ORDER BY prov.name';
  $result = db_query($sql, $set_id);
  $providers = array();
  while ($data = db_fetch_object($result)) {
    $providers[] = l(check_plain($data->name),
      'admin/xc/harvester/repository/' . check_plain($data->provider_id));
  }
  return $providers;
}

/**
 * Get repository by an URL. There should be only one repository for an URL.
 * @param $url The URL
 * @return unknown_type
 */
function _oaiharvester_provider_get_by_url($url) {
  $sql = 'SELECT * from {oaiharvester_providers} WHERE oai_provider_url = \'%s\'';
  $result = db_query($sql, $url);
  $data = db_fetch_object($result);
  return $data;
} // _oaiharvester_get_repository

/**
 * Get all provider’s name
 * @return unknown_type
 */
function oaiharvester_provider_get_all_names() {
  $providers = array();
  $sql = 'SELECT * from {oaiharvester_providers}';
  $result = db_query($sql);
  while ($data = db_fetch_object($result)) {
    $providers[$data->provider_id] = $data->name;
  }
  return $providers;
}

function _oaiharvester_provider_get_url_by_id($provider_id) {
  static $provider_cache;

  if (!isset($provider_cache[$provider_id])) {
    $sql = 'SELECT oai_provider_url from {oaiharvester_providers} WHERE provider_id = %d';
    $result = db_query($sql, $provider_id);
    $data = db_fetch_object($result);
    $provider_cache[$provider_id] = $data->oai_provider_url;
  }
  return $provider_cache[$provider_id];
} // _oaiharvester_get_repository

function _oaiharvester_schedule_steps_delete($harvest_schedule_id) {
  $sql = 'DELETE FROM {oaiharvester_harvest_schedule_steps}
          WHERE schedule_id = %d';
  $result = db_query($sql, $harvest_schedule_id);
}

function _oaiharvester_schedule_steps_delete_by_stepid($harvest_schedule_steps_id) {
  $sql = 'DELETE FROM {oaiharvester_harvest_schedule_steps}
          WHERE harvest_schedule_steps_id = %d';
  $result = db_query($sql, $harvest_schedule_steps_id);
}

function _oaiharvester_schedule_step_delete_object($step) {
  _oaiharvester_schedule_step_delete($step->schedule_id, $step->format_id, $step->set_id);
}

function _oaiharvester_schedule_step_delete($schedule_id, $format_id, $set_id) {
  if (empty($set_id)) {
    $sql = 'DELETE FROM {oaiharvester_harvest_schedule_steps}
               WHERE schedule_id = %d
                 AND format_id = %d
                 AND set_id IS NULL';
    $result = db_query($sql, $schedule_id, $format_id);
  }
  else {
    $sql = 'DELETE FROM {oaiharvester_harvest_schedule_steps}
               WHERE schedule_id = %d
                 AND format_id = %d
                 AND set_id = %d';
    $result = db_query($sql, $schedule_id, $format_id, $set_id);
  }
}

/**
 * Set schedule state to active (running)
 * @param $schedule_id (int) The ID of the schedule
 * @return (Boolean) Flag of action’s success (true/false)
 */
function _oaiharvester_schedule_set_active($schedule_id) {
  return _oaiharvester_schedule_set_state($schedule_id, OAIHARVESTER_ACTIVE);
}

/**
 * Set schedule state to passive (not running)
 * @param $schedule_id (int) The ID of the schedule
 * @return (Boolean) Flag of action’s success (true/false)
 */
function _oaiharvester_schedule_set_passive($schedule_id) {
  return _oaiharvester_schedule_set_state($schedule_id, OAIHARVESTER_PASSIVE);
}

/**
 * Set the state of the schedule (active or passive)
 * @param $schedule_id (int) The ID of the schedule
 * @param $state (String) The state of the schedule
 * @return (Boolean) Flag of action’s success (true/false)
 */
function _oaiharvester_schedule_set_state($schedule_id, $state) {
  if (!in_array($state, array(OAIHARVESTER_ACTIVE, OAIHARVESTER_PASSIVE))) {
    return FALSE;
  }

  $schedule = _oaiharvester_schedule_get($schedule_id);
  $schedule->state = $state;
  $ret_val = drupal_write_record('oaiharvester_harvester_schedules', $schedule,
    'schedule_id');
  if ($ret_val == SAVED_UPDATED) {
    return TRUE;
  }
  return FALSE;
}

function _oaiharvester_schedule_steps_get($schedule_id) {
  $sql = 'SELECT step.harvest_schedule_steps_id, step.last_ran, fmt.*, sets.*
            FROM {oaiharvester_harvest_schedule_steps} AS step
              INNER JOIN {oaiharvester_formats} AS fmt
                    ON fmt.format_id = step.format_id
               LEFT JOIN {oaiharvester_sets} AS sets
                    ON sets.set_id = step.set_id
                 WHERE schedule_id = %d';
  $result = db_query($sql, $schedule_id);
  $steps = array();
  while ($data = db_fetch_object($result)) {
    $steps[] = $data;
  }
  return $steps;
}

function _oaiharvester_schedule_steps_get_atomic($schedule_id) {
  $sql = 'SELECT harvest_schedule_steps_id, schedule_id, format_id, set_id
            FROM {oaiharvester_harvest_schedule_steps} WHERE schedule_id = %d';
  $result = db_query($sql, $schedule_id);
  $steps = array();
  while ($data = db_fetch_object($result)) {
    $steps[] = $data;
  }
  return $steps;
}

/**
 * Reset the last ran value to null for a given schedule step
 * @param $step_id
 * @return unknown_type
 */
function _oaiharvester_schedule_step_reset($step_id) {
  $sql = 'UPDATE {oaiharvester_harvest_schedule_steps}
             SET last_ran = NULL
             WHERE harvest_schedule_steps_id = %d';
  $result = db_query($sql, $step_id);
  return $result;
}

function _oaiharvester_schedule_step_get_atomic($schedule_id, $format_name, $set_spec = NULL) {
  $sql = 'SELECT step.*
            FROM {oaiharvester_harvest_schedule_steps} step
            INNER JOIN {oaiharvester_formats} AS format
                    ON format.format_id = step.format_id';
  if ($set_spec != NULL) {
    $sql .= ' LEFT JOIN {oaiharvester_sets} AS sets
                    ON sets.set_id = step.set_id';
  }
  $sql .= ' WHERE schedule_id = %d
              AND format.name      = \'%s\'';
  if ($set_spec != NULL) {
    $sql .= ' AND sets.set_spec  = \'%s\'';
  }
  else {
    $sql .= ' AND step.set_id IS NULL';
  }
  $result = db_query($sql, $schedule_id, $format_name, $set_spec);
  $steps = array();
  while ($data = db_fetch_object($result)) {
    $steps[] = $data;
  }
  return $steps;
}

/**
 * Get schedule by id
 * @param $id The ID of schedule
 * @return stdClass The schedule object
 */
function _oaiharvester_schedule_get($id) {
  static $cache;
  if (!isset($cache[$id])) {
    $sql = 'SELECT * from {oaiharvester_harvester_schedules} WHERE harvest_schedule_id = %d';
    $result = db_query($sql, $id);
    $data = db_fetch_object($result);
    $cache[$id] = $data;
  }
  return $cache[$id];
} // _oaiharvester_schedule_get

/**
 * Get schedule by provider id
 * @param $id (int)
 *   The ID of data provider
 * @return (Array)
 *   List of links to schedules
 */
function _oaiharvester_schedule_get_by_provider($provider_id) {
  static $cache;
  if (!isset($cache[$provider_id])) {
    $sql = 'SELECT * FROM {oaiharvester_harvester_schedules} WHERE provider_id = %d';
    $result = db_query($sql, $provider_id);
    $links = array();
    while ($data = db_fetch_object($result)) {
      $links[] = l($data->schedule_name, 'admin/xc/harvester/schedule/' . $data->harvest_schedule_id);
    }
    $cache[$provider_id] = $links;
  }
  return $cache[$provider_id];
} // _oaiharvester_schedule_get

/**
 * Get all schedule data, and the provider name and URL
 * @param $schedule_id The schedule ID
 */
function _oaiharvester_schedule_get_full($schedule_id) {
  static $cache;

  if (!isset($cache[$schedule_id])) {
    $sql = 'SELECT schedule.*,
                   provider.name AS repository_name,
                   provider.oai_provider_url,
                   provider.oai_identifier,
                   (DATE(end_date) >= CURDATE()) AS active
            FROM {oaiharvester_harvester_schedules} AS schedule
              LEFT JOIN {oaiharvester_providers} AS provider
                     ON provider.provider_id = schedule.provider_id
            WHERE schedule.harvest_schedule_id = %d';
    $result = db_query($sql, $schedule_id);
    $data = db_fetch_object($result);
    if (!is_null($data)) {
      $data->oai_identifier = unserialize($data->oai_identifier);
    }
    $cache[$schedule_id] = $data;
  }
  return $cache[$schedule_id];
} // _oaiharvester_schedule_get_full

/**
 * Delete a schedule from the database
 * @param $schedule_id The identifier of the schedule
 * @return unknown_type
 */
function _oaiharvester_schedule_delete($schedule_id) {
  $sql = 'DELETE FROM {oaiharvester_harvester_schedules}
          WHERE harvest_schedule_id = %d';
  return db_query($sql, $schedule_id);
} // _oaiharvester_schedule_delete

/**
 * Get sets of a provider
 * @param $provider_id The ID of provider
 * @return resource Database query result resource or FALSE
 */
function _oaiharvester_sets_get_by_provider($provider_id) {
  static $cache;

  if (!isset($cache[$provider_id])) {
    $sql = 'SELECT sets.* '
         .   'FROM {oaiharvester_sets} AS sets, '
         .        '{oaiharvester_sets_to_providers} AS con'
         . ' WHERE sets.set_id = con.set_id'
         .   ' AND con.provider_id = %d';
    $result = db_query($sql, $provider_id);
    $cache[$provider_id] = $result;
  }
  return $cache[$provider_id];
} // _oaiharvester_sets_get_by_provider

/**
 * Get the metadata formats by a provider
 * @param $provider_id The ID of a provider
 * @return resource Database query result resource or FALSE
 */
function _oaiharvester_metadataformats_get_by_provider($provider_id) {
  $sql = 'SELECT formats.*
           FROM {oaiharvester_formats} AS formats,
                {oaiharvester_formats_to_providers} AS con'
       . ' WHERE formats.format_id = con.format_id'
       .   ' AND con.provider_id = %d';
  return db_query($sql, $provider_id);
} // _oaiharvester_metadataformats_get_by_provider

/**
 * Get the whole queue
 * @return unknown_type
 */
function _oaiharvester_queue_get() {
  $sql = 'SELECT * FROM {oaiharvester_harvest_queue}';
  return db_query($sql);
} // _oaiharvester_queue_get

/**
 * Get a queue item by its ID
 * @param $queue_id
 * @return unknown_type
 */
function _oaiharvester_queue_get_item_by_id($queue_id) {
  $sql = 'SELECT * FROM {oaiharvester_harvest_queue} WHERE harvest_id = %d';
  xc_log_info('harvester', 'queue_item_by_id: ' . xc_util_get_sql($sql, $queue_id));
  return db_fetch_object(db_query($sql, $queue_id));
} // _oaiharvester_queue_get_item_by_id

function _oaiharvester_queue_delete_item_by_id($queue_id) {
  $sql = 'DELETE FROM {oaiharvester_harvest_queue} WHERE harvest_id = %d';
  return db_query($sql, $queue_id);
} // _oaiharvester_queue_delete_item_by_id

function _oaiharvester_queue_delete_all() {
  return db_query('DELETE FROM {oaiharvester_harvest_queue}');
} // _oaiharvester_queue_delete_all

/**
 * Return a queue item connected to a given schedule
 * @param $schedule_id (int)
 *   The schedule ID
 * @param $format (String)
 *   The metadata format’s name to harvest
 * @param $set (String)
 *   The set spec to harvest
 * @return
 *   A database query result resource, or FALSE if the query was not executed
 *   correctly.
 */
function _oaiharvester_queue_get_item_by_schedule($schedule_id, $format_name,
    $set_spec) {
  $sql = 'SELECT * FROM {oaiharvester_harvest_queue}
            WHERE harvest_schedule_id = %d
              AND metadata_prefix = \'%s\'';
  if (!empty($set)) {
    $sql .= ' AND set_name = \'%s\'';
    return db_query($sql, $schedule_id, $format_name, $set_spec);
  }
  $sql .= ' AND set_name IS NULL';
  return db_query($sql, $schedule_id, $format_name);
} // _oaiharvester_queue_get_item_by_schedule
