<?php
/**
 * @file
 * The collection of reindexing functions
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

/**
 * A confirmation form for reindexing a given data source
 *
 * @return
 *   A confirmation form
 */
function xc_index_reindex_form() {
  include_once(drupal_get_path('module', 'xc_metadata') . '/includes/xc_metadata.source.inc');
  $sources = xc_source_get_all();
  $source_options = array();
  foreach ($sources as $source) {
    $source_options[$source->source_id] = $source->name;
  }
  $form['description'] = array(
    '#type' => 'item',
    '#title' => t('Usage notes'),
    '#value' => t('You should reindex existing metadata when you changed the indexing settings, but you do not want harvest your source again. You should be very careful with this action, because it totally regenerate your Solr index so it is always suggestable to save your existing index before launching reindex.'),
  );

  $form['source_id'] = array(
    '#type' => 'radios',
    '#title' => 'Metadata sources',
    '#description' => 'Please select which metadata source you would like to reindex',
    '#options' => $source_options,
  );

  return confirm_form(
    array($form),
    t('Are you sure, that you would like reindex the metadata?'),
    'admin/xc/index', // path to go if user click on 'cancel'
    t('This action cannot be undone.'),
    t('Start process'),
    t('Cancel')
  );
}

/**
 * Submission handling funtion for the reindex confirmation form
 *
 * @param $form (Array)
 *   The FAPI object for the form itself
 * @param $form_state (Array)
 *   The FAPI object for the data
 */
function xc_index_reindex_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  if ($values['confirm'] === 1
      && isset($values['source_id'])
      && intval($values['source_id']) > 0) {
    xc_index_reindex_launch(intval($values['source_id']));
  }
  else {
    $form_state['redirect'] = 'admin/xc/index';
  }
}

/**
 * Creates and launch a batch process which reindex the source, and
 * make the metadata normalization (de-rfbr-ization) for preparing search
 *
 * @param $source_id (int)
 *   The source identifier (see xc_source table)
 */
function xc_index_reindex_launch($source_id = NULL) {
  include_once(drupal_get_path('module', 'xc_metadata') . '/includes/xc_metadata.location.inc');
  $xc_location = xc_location_get_by_source($source_id);
  $operations = array(
    array('xc_index_reindex', array($source_id, array($xc_location))),
    array('xc_solr_defrbrzize_index', array($xc_location, 'manual', NULL, 0))
  );
  $batch = array(
    'operations'       => $operations,
    'title'            => t('Reindexing Solr'),
    'init_message'     => t('Initialize reindexing process'),
    'progress_message' => t('Keep on running'),
    'finished'         => 'xc_index_reindex_finished',
  );
  batch_set($batch);
  batch_process('admin/xc/index');
}

