<?php
/**
 * @file
 * XC NCIP provider authentication functions
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

/**
 * Implementation of hook_xc_auth_methods()
 */
function xc_ncip_provider_xc_auth_methods() {
  $methods['ncip_provider'] = array(
    'name' => t('NCIP server authentication'),
    'description' => t('Authentication with NCIP server using either username and password or LDAP username and password'),
    'credentials' => 'user_pass'
  );

  return $methods;
}

/**
 * Implementation of xc_auth_method_hook_form()
 */
function xc_auth_method_ncip_provider_form(&$form_state) {
  $ncip_provider_options = array();
  $ncip_providers = xc_ncip_provider_get_all();

  foreach ($ncip_providers as $ncip_provider) {
    if ($ncip_provider->properties['auth']['allow_authentication']) {
      $ncip_provider_options[$ncip_provider->ncip_provider_id] = $ncip_provider->name;
    }
  }

  $form['ncip_provider_auth_type'] = array(
    '#type' => 'radios',
    '#title' => t('NCIP server authentication type'),
    '#description' => t('Whether NCIP authenticating by default or LDAP username and password'),
    '#options' => array(
      'default' => t('Default'),
      'ldap' => t('LDAP')
    ),
    '#default_value' => 'default',
    '#required' => TRUE
  );

  $form['ncip_provider_id'] = array(
    '#type' => 'select',
    '#title' => t('NCIP server'),
    '#options' => $ncip_provider_options,
    '#required' => TRUE
  );

  return $form;
}

function xc_auth_method_ncip_provider_validate($form, &$from_state) {}

function xc_auth_method_ncip_provider_submit($form, &$from_state) {}

function xc_auth_method_login_ncip_provider_validate($form, &$from_state) {}

function xc_auth_method_login_ncip_provider_submit($form, &$from_state) {}

/**
 * Implementation of xc_auth_method_hook_xc_account()
 */
function xc_auth_method_ncip_provider_xc_account($op, $account, $auth_user, $auth_type) {
  global $user;
  static $cache;

  // Return if current logged in user is not the same as account
  if ($user->uid != $account->uid) {
    xc_log_error('xc_auth', 'Current logged in user is not the same as account.');
    return array();
  }

  // Get user Id from the current session or return
  // TODO: problem: $auth_type->tid is stored in DB independently from the session
  if (!$user_id = $_SESSION['xc_auth_types'][$auth_type->tid]['data']['ncip_provider']['UserIdentifierValue']) {
    xc_log_error('xc_auth', 'There is no user id in the current session for %name.', array('%name' => $auth_type->name));
    return array();
  }

  // NCIP provider Id from the authentication type
  $ncip_provider_id = $auth_type->data['ncip_provider']['ncip_provider_id'];

  // If NCIP provider has already been done for this user in this page load
  // then don't do it again
  if ($cache[$account->uid][$ncip_provider_id]) {
    xc_log_error('xc_auth', 'NCIP provider has already been done for this user in this page load');
    return array();
  }
  else {
    $cache[$account->uid][$ncip_provider_id] = TRUE;
  }

  $results = xc_ncip_provider_lookup_user($ncip_provider_id, $user_id,
      array('User Address Information', 'Visible User Id'), TRUE, TRUE, TRUE);
  if ($results['error'] || !$results['results']) {
    // A problem has occurred
    xc_log_error('xc_auth', 'Problem occured during NCIP account call: %error', array('%error' => $results['error']));
    return array();
  }
  else {
    switch ($op) {
      case 'personal info':
        // Name
        if ($results['results'][$user_id]['FullName']) {
          $personal_info['name'] = $results['results'][$user_id]['FullName'];
        }

        // Address
        if (is_array($results['results'][$user_id]['Address']['Physical'])) {
          foreach ($results['results'][$user_id]['Address']['Physical'] as $physical_address) {
            $personal_info['address'][] = array(
              'type' => $physical_address['Type'],
              'address' => $physical_address['Address']
            );
          }
        }

        // Email
        if (is_array($results['results'][$user_id]['Address']['Electronic'])) {
          foreach ($results['results'][$user_id]['Address']['Electronic'] as $email) {
            $personal_info['email'][] = array(
              'type' => $email['Type'],
              'email' => $email['Address']
            );
          }
        }

        // Totals -- loaned and requested items and account balance
        if (is_array($results['results'][$user_id]['LoanedItems'])) {
          $personal_info['num_loaned_items'] = count($results['results'][$user_id]['LoanedItems']);
        }
        if (is_array($results['results'][$user_id]['RequestedItems'])) {
          $personal_info['num_requested_items'] = count($results['results'][$user_id]['RequestedItems']);
        }
        if (is_array($results['results'][$user_id]['FiscalAccount']['Balance'])) {
          $personal_info['account_balance'] = ncip_display_amount(
            $results['results'][$user_id]['FiscalAccount']['Balance']['CurrencyCode'],
            $results['results'][$user_id]['FiscalAccount']['Balance']['MonetaryValue']
          );
        }
        return $personal_info;
        break;

      case 'loaned items':
        $items = array();
        if (is_array($results['results'][$user_id]['LoanedItems'])) {
          $item_ids = array();
          foreach ($results['results'][$user_id]['LoanedItems'] as $item) {
            $item_ids[] = $item['ItemId'];
          }

          // Get availabliity
          $availability = xc_ncip_provider_xc_get_availability($ncip_provider_id,
            array('item' => $item_ids));

          foreach ($results['results'][$user_id]['LoanedItems'] as $item) {
            $items[] = array(
              'type' => 'ncip_provider',
              'ncip_provider_id' => $ncip_provider_id,
              'item_id' => $item['ItemId'],
              'bib_id' => (isset($availability['results'])
                ? $availability['results']['item'][$item['ItemId']][0]['BibId']
                : NULL),
              'date_due' => @strtotime($item['DateDue']),
              'bib_desc' => (isset($availability['results'])
                ? $availability['results']['item'][$item['ItemId']][0]['BibliographicDescription']
                : NULL),
              'renew' => array(
                'path' => 'admin/xc/ncip/provider/' . $ncip_provider_id . '/renew/' . $user_id . '/' . $item['ItemId'],
                'options' => array('query' => 'destination=' . $_GET['q'])
              )
            );
          }
        }
        return $items;
        break;

      case 'requested items':
        $items = array();
        if (is_array($results['results'][$user_id]['RequestedItems'])) {
          $item_ids = array();
          foreach ($results['results'][$user_id]['RequestedItems'] as $item) {
            $item_ids[] = $item['ItemId'];
          }

          // Get availabliity
          $availability = xc_ncip_provider_xc_get_availability($ncip_provider_id,
            array('item' => $item_ids));

          foreach ($results['results'][$user_id]['RequestedItems'] as $item) {
            $items[] = array(
              'type' => 'ncip_provider',
              'ncip_provider_id' => $ncip_provider_id,
              'item_id' => $item['ItemId'],
              'bib_id' => (isset($availability['results'])
                ? $availability['results']['item'][$item['ItemId']][0]['BibId']
                : NULL),
              'bib_desc' => (isset($availability['results'])
                ? $availability['results']['item'][$item['ItemId']][0]['BibliographicDescription']
                : NULL),
              'date_due' => @strtotime($item['DateDue']),
              'date_placed' => @strtotime($item['DatePlaced'])
            );
          }
        }
        return $items;
        break;
    }
  }
}

/**
 * Implementation of hook_xc_login()
 */
function xc_ncip_provider_xc_login($auth, $values, &$data) {
  switch ($auth->method) {
    case 'ncip_provider':
      $result = xc_ncip_provider_authenticate_user(
        $auth->data['ncip_provider']['ncip_provider_id'],
        $values,
        $auth->data['ncip_provider']['ncip_provider_auth_type']
      );

      if ($result['results']) {
        // The user is authenticated, now keep track of user identifier number
        $data['ncip_provider']['UserIdentifierValue'] = current($result['results']);
        return XC_AUTH_LOGIN_SUCCESS;
      }
      elseif ($result['error']) {
        switch ($result['error']['ErrorCode']) {
          // Only occurs because of incorrect credentials
          case '0301 - Failed Login':
            return XC_AUTH_LOGIN_INCORRECT;
            break;
          default:
            return XC_AUTH_LOGIN_FAILURE;
            break;
        }
      }
      else {
        return XC_AUTH_LOGIN_FAILURE;
      }
      break;
  }
}

/**
 * Implementation of hook_xc_check_login()
 */
function xc_ncip_provider_xc_check_login($auth, $values = array(), $identity = '', $refresh = FALSE) {
  switch ($auth->method) {
    case 'ncip_provider':
      // Make sure the NCIP provider is loaded
      global $_xc_ncip_provider;
      xc_ncip_provider_start();

      $ncip_provider_id = $auth->data['ncip_provider']['ncip_provider_id'];
      // TODO: Need a real way of checking the authenticated status with the
      // NCIP provider. Also need to supress debugger or it will keep track of
      // all user passwords submitted over NCIP

      // If NCIP provider connection is active, then assume the user is
      // authenticated / logged in otherwise they are no longer logged into
      // the NCIP provider (and the ILS or LDAP server)
      $connection_id = $_SESSION['xc_ncip_provider_connections'][$ncip_provider_id];
      if ($_xc_ncip_provider->load_connection($connection_id)) {
        return TRUE;
      }
      else {
        return FALSE;
      }
      break;

      // ANOTHER METHOD:
      // Use LookupUser to see if financial information is in response message to
      // test if logged in or not. This may not work every time, but it is more
      // reliable than the previous method of checking the connection
      //
      // $user_id = $_SESSION['xc_auth_types'][$auth->tid]['data']['ncip_provider']
      //   ['UserIdentifierValue'];
      // $response = xc_ncip_provider_lookup_user($ncip_provider_id, $user_id,
      //   array(), FALSE, FALSE, TRUE, 'message');
      // return isset($response->message[$response->service]['UserFiscalAccount']);
  }
}

/**
 * Implementation of hook_xc_logout()
 */
function xc_ncip_provider_xc_logout($auth, $data) {
  switch ($auth->method) {
    case 'ncip_provider':
      // Close the NCIP connection... a new connection will be made
      // automatically by the NCIP provider module, when needed
      return xc_ncip_provider_disconnect($auth->data['ncip_provider']['ncip_provider_id']);
      break;
  }
}

