<?php
/**
 * @file
 * Functions related to Solr indexing
 *
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

/**
 * For multiple value sortable field index the first instance
 * @var int
 */
define('XC_SOLR_SORT_STRATEGY_FIRST',  2);

/**
 * For multiple value sortable field index the first minimal and the maximal values
 * @var int
 */
define('XC_SOLR_SORT_STRATEGY_MINMAX', 1);

/**
 * Metadata process strategy: merge all metadata first then call indexing
 * @var int
 */
define('XC_SOLR_METADATA_PROCESS_STRATEGY_MERGE_FIRST',  1);

/**
 * Metadata process strategy: index metadata as soon as possible
 * @var int
 */
define('XC_SOLR_METADATA_PROCESS_STRATEGY_INDEX_FIRST',  2);

/**
 * Displays the schema field to Solr fields mapping
 */
function xc_solr_show_fields2index_list() {
  global $_xc_solr_obj;

  variable_del('xc_solr_fields2index_list');
  $list = xc_solr_fields2index_list();

  $rows = array();
  foreach ($list as $schema_field => $targets) {
    $cells = array();
    $cells[] = $schema_field;
    $output = array();
    foreach ($targets as $type => $solr_fields) {

      if ($type == 'value_as_key') {
        // array ( 'value_as_key' => array ( 'type' => array ( 'values' => array ( '@gtt ' => array ( 0 => 'xc__subject_gtt _t', ), ), ), ),
        foreach ($solr_fields as $attribute => $properties) {
          foreach ($properties['values'] as $property => $solr_fields2) {
            $output[] = '@' . $attribute . '=' . $property . ': ' . join(', ', $solr_fields2);
          }
        }
      }
      else {
        $output[] = $type . ': ' . join(', ', $solr_fields);
      }
    }
    $cells[] = join('<br/>', $output);

    $rows[] = $cells;
  }
  return theme('table', array(t('Schema field'), t('Solr fields')), $rows);
}

/**
 * Displays a confirmation form for launching the one-stop Solr indexer
 *
 * @param $form_state (Array)
 *   The FAPI form state array
 */
function xc_solr_onestop_indexer_batch_launcher_form(&$form_state) {
  // TODO: check whether Solr is runing
  $form['xc_location'] = array(
    '#type' => 'value',
    '#value' => serialize($xc_location)
  );

  return confirm_form(
    array($form),
    t('Are you sure, that you would like to launch indexing metadata for search?'),
    'admin/xc/solr', // path to go if user click on 'cancel'
    t('This action cannot be undone.'),
    t('Index metadata'),
    t('Cancel')
  );
}

/**
 * Submission handler for the confirmation form for launching the one-stop Solr indexer.
 *
 * @param $form (Array)
 *   The FAPI form array
 * @param $form_state (Array)
 *   The FAPI form state array
 */
function xc_solr_onestop_indexer_batch_launcher_form_submit($form, &$form_state) {
  global $_oaiharvester_statistics;

  $xc_location = unserialize($form_state['values']['xc_location']);
  variable_del('oaiharvester_statistics');
  variable_del('xc_solr_fields2index_list');
  variable_del('xc_solr_sortable_fields');
  $_oaiharvester_statistics = array();

  xc_solr_onestop_indexer_batch_launcher();
}

function xc_solr_onestop_indexer_batch_launcher() {
  drupal_set_message('xc_solr_onestop_indexer_batch_launcher');
  if (!function_exists('xc_location_get')) {
    include_once drupal_get_path('module', 'xc_metadata') . '/xc_metadata.location.inc';
  }
  variable_set('xc_solr_fields2index_list_applied', xc_solr_fields2index_list('simple'));
  $locations = xc_location_get(1);
  $batch = array(
    'operations'       => array(array('xc_solr_onestop_indexer_batch', array(NULL, array($locations), -1, -1))),
    'title'            => t('Indexing Solr'),
    'init_message'     => t('Initialize reindexing process'),
    'progress_message' => t('Keep on running'),
    'finished'         => 'xc_solr_onestop_indexer_batch_finished',
  );
  batch_set($batch);
  batch_process('admin/xc/solr'); // index?
}

function xc_solr_onestop_indexer($metadata_id) {
  global $_oaiharvester_statistics, $_xc_solr_conditional_facets;
  static $commons;

  if (!isset($commons)) {
    $commons = new stdClass();
  }

  if (!isset($commons->metadata_process_strategy)) {
    $commons->metadata_process_strategy = xc_solr_get_metadata_process_strategy();
  }

  if (!isset($commons->conflicting_fields)) {
    $commons->conflicting_fields = xc_solr_get_conflicting_fields();
    if (!empty($commons->conflicting_fields)) {
      $commons->indexes = xc_solr_fields2index_list();
      $commons->conflicting_fields_cache = array();
    }
  }

  $_xc_solr_conditional_facets = array();
  // iterate manifestations
  $manifestation = xc_solr_get_metadata($metadata_id);
  $metadata = $manifestation->metadata;
  if ($commons->metadata_process_strategy == XC_SOLR_METADATA_PROCESS_STRATEGY_INDEX_FIRST) {
    $document = xc_solr_metadata_to_solr_iterate_indexes($manifestation);
  }
  else if (isset($commons->conflicting_fields['manifestation'])) {
    foreach ($commons->conflicting_fields['manifestation'] as $schema_field => $solr_field) {
      if ($commons->indexes[$schema_field]['value_as_key']) {
        $commons->conflicting_fields_cache[$solr_field] = xc_solr_index_general_value_as_key($metadata,
          $schema_field, $commons->indexes[$schema_field]);
      }
    }
  }

  // iterate holdings
  $holdings_ids = xc_entity_get_child_identifiers($manifestation->identifier_int);
  foreach ($holdings_ids as $holdings_identifier_int) {
    $holdings = xc_solr_get_metadata_by_identifier_int($holdings_identifier_int);

    if (is_array($holdings->metadata)) {
      $ft1 = microtime(TRUE);
      $metadata = array_merge_recursive($metadata, $holdings->metadata);
      $ft2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/01 metadata merge'] += $ft2 - $ft1;
    }

    if ($commons->metadata_process_strategy == XC_SOLR_METADATA_PROCESS_STRATEGY_INDEX_FIRST) {
      xc_solr_metadata_to_solr_iterate_indexes($holdings, $document);
    }
    else if (isset($commons->conflicting_fields['holdings'])) {
      foreach ($commons->conflicting_fields['holdings'] as $schema_field => $solr_field) {
        if ($commons->indexes[$schema_field]['value_as_key']) {
          $commons->conflicting_fields_cache[$solr_field] = xc_solr_index_general_value_as_key($holdings->metadata,
            $schema_field, $commons->indexes[$schema_field]);
        }
      }
    }
  }

  // $expression_ids = xc_solr_get_parent($manifestation);
  $expressions = xc_solr_get_parent($manifestation);

  foreach ($expressions as $expression) {
    if ($expression === FALSE) {
      xc_log_error('solr', 'empty expression in: ' . var_export($expressions, TRUE) . ', ' . var_export($manifestation->metadata, TRUE));
      continue;
    }

    if (is_array($expression->metadata)) {
      $ft1 = microtime(TRUE);
      $metadata = array_merge_recursive($metadata, $expression->metadata);
      $ft2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/01 metadata merge'] += $ft2 - $ft1;
    }

    if ($commons->metadata_process_strategy == XC_SOLR_METADATA_PROCESS_STRATEGY_INDEX_FIRST) {
      xc_solr_metadata_to_solr_iterate_indexes($expression, $document);
    }
    else if (isset($commons->conflicting_fields['expression'])) {
      foreach ($commons->conflicting_fields['expression'] as $schema_field => $solr_field) {
        if ($commons->indexes[$schema_field]['value_as_key']) {
          $commons->conflicting_fields_cache[$solr_field] = xc_solr_index_general_value_as_key($expression->metadata,
            $schema_field, $commons->indexes[$schema_field]);
        }
      }
    }
    $works = xc_solr_get_parent($expression);

    foreach ($works as $work) {
      if ($work === FALSE) {
        xc_log_error('solr', 'empty work in: ' . var_export($works, TRUE));
        continue;
      }

      if (is_array($work->metadata)) {
        $ft1 = microtime(TRUE);
        $metadata = array_merge_recursive($metadata, $work->metadata);
        $ft2 = microtime(TRUE);
        $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/01 metadata merge'] += $ft2 - $ft1;
      }

      if ($commons->metadata_process_strategy == XC_SOLR_METADATA_PROCESS_STRATEGY_INDEX_FIRST) {
        xc_solr_metadata_to_solr_iterate_indexes($work, $document);
      }
      else if (isset($commons->conflicting_fields['work'])) {
        foreach ($commons->conflicting_fields['work'] as $schema_field => $solr_field) {
          if ($commons->indexes[$schema_field]['value_as_key']) {
            $commons->conflicting_fields_cache[$solr_field] = xc_solr_index_general_value_as_key($work->metadata,
              $schema_field, $commons->indexes[$schema_field]);
          }
        }
      }
    }
  }

  if ($commons->metadata_process_strategy == XC_SOLR_METADATA_PROCESS_STRATEGY_MERGE_FIRST) {
    $manifestation->metadata = $metadata;
    $document = xc_solr_metadata_to_solr_iterate_indexes($manifestation);

    // adding conflicting fields
    if (!empty($commons->conflicting_fields_cache)) {
      foreach ($commons->conflicting_fields_cache as $solr_field => $values) {
        $document->addField($solr_field, $values, FALSE, TRUE);
      }
    }
  }

  $document->addField('metadata_tsn',  serialize($metadata), FALSE, TRUE);
  $document->setField('text', join(' ', $document->text));

  $ft1 = microtime(TRUE);

  // format facet (it is a special case of facets)
  // TODO: convert this to normal hook or DB based conditions
  xc_index_format_facet($document, $metadata);
  $ft2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/02 format facet'] += $ft2 - $ft1;

  // general case for facet hooks
  if (!empty($_xc_solr_conditional_facets)) {
    foreach ($_xc_solr_conditional_facets as $facet => $fields) {
      if (!isset($commons->field_hooks)) {
        $commons->field_hooks = variable_get('xc_solr_field_hooks', array());
      }
      if (!isset($commons->field_hooks[$facet])) {
        $commons->field_hooks[$facet] = array();
        $hook = 'xc_process_facet_' . $facet;
        foreach (module_implements($hook) as $module) {
          $commons->field_hooks[$facet][] = $module . '_' . $hook;
        }
        variable_set('xc_solr_field_hooks', $commons->field_hooks);
      }
      foreach ($commons->field_hooks[$facet] as $function) {
        $function($document, $metadata, array_keys($fields));
      }
    }
  }
  $ft3 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/03 conditional facets 1 - hooks'] += $ft3 - $ft2;

  // general case for conditional facet existing in database
  if (!isset($commons->conditional_facets)) {
    require_once drupal_get_path('module', 'xc_index') . '/xc_index_facet.inc';
    $commons->conditional_facets = xc_index_facet_list_conditionals(TRUE);
    $commons->conditional_facets_file_required = FALSE;
  }

  if (!empty($commons->conditional_facets)) {
    foreach ($commons->conditional_facets as $facet) {
      if ($facet->fn !== FALSE) {
        // require necessary file
        if (!$commons->conditional_facets_file_required) {
          require_once drupal_get_path('module', 'xc_index') . '/xc_index_facet_mapping.inc';
          $commons->conditional_facets_file_required = TRUE;
        }

        $mapping = xc_index_facet_for_edit_load($facet->name);
        $function = $facet->fn;
        $values = $function(&$document, $metadata, $mapping->fields);
      }
    }
  }
  $ft4 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/04 conditional facets 2 - forms'] += $ft4 - $ft3;

  if (!isset($commons->sortable_fields)) {
    $commons->sortable_fields = variable_get('xc_solr_sortable_fields', NULL);
    if (is_null($commons->sortable_fields)) {
      if (!function_exists('xc_index_sortables_get_all')) {
        require_once drupal_get_path('module', 'xc_index') . '/xc_index_sortables.inc';
      }
      $fields = xc_index_sortables_get_all();
      if (!empty($fields)) {
        $suffixes = array_diff(array_keys(xc_solr_field_type_get_types()), array('n/a'));
        $regex = '/^(.*?)(' .  join('|', $suffixes) . ')$/';
        $commons->sortable_fields = array();
        foreach ($fields as $field) {
          if (preg_match($regex, $field, $matches)) {
            $commons->sortable_fields[$field] = array('name' => $matches[1], 'suffix' => $matches[2]);
          }
          else {
            $commons->sortable_fields[$field] = array('name' => $field, 'suffix' => '');
          }
        }
      }
      else {
        $commons->sortable_fields = array();
      }
      variable_set('xc_solr_sortable_fields', $commons->sortable_fields);
    }
  }

  if (!empty($commons->sortable_fields)) {
    $sort_strategy = xc_solr_get_sortstrategy();
    foreach ($commons->sortable_fields as $field => $properties) {
      if (isset($document->$field)) {
        $instances = $document->$field;
        if (!empty($instances)) {
          $suffix = ($properties['suffix'] == '_dt' || $properties['suffix'] == '_df') ? '_sdt' : '_ss';
          $len = count($instances) - 1;
          if ($sort_strategy == XC_SOLR_SORT_STRATEGY_MINMAX) {
            sort($instances);
            for ($i = 0; $i <= $len; $i++) {
              if ($instances[$i] != '') {
                $document->addField($properties['name'] . '_min' . $suffix, $instances[$i]);
                break;
              }
            }
            for ($i = $len; $i >= 0; $i--) {
              if ($instances[$i] != '') {
                $document->addField($properties['name'] . '_max' . $suffix, $instances[$i]);
                break;
              }
            }
          }
          elseif ($sort_strategy == XC_SOLR_SORT_STRATEGY_FIRST) {
            for ($i = 0; $i <= $len; $i++) {
              if ($instances[$i] != '') {
                $document->addField($properties['name'] . $suffix, $instances[$i]);
                break;
              }
            }
          }
        }
      }
    }
  }
  $ft5 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/05 sortable fields'] += $ft5 - $ft4;

  return $document;
}

function xc_solr_get_metadata($metadata_id) {
  global $_oaiharvester_statistics;
  static $sql;

  $t = microtime(TRUE);
  if (empty($metadata_id)) {
    return FALSE;
  }
  // xc_log_info('solr', 'get #' . $metadata_id);
  if (!isset($sql)) {
    $sql = 'SELECT e.identifier, e.identifier_int, e.node_id, e.node_type, e.metadata_id, e.metadata_type, e.source_id,
            m.metadata AS metadata
            FROM {xc_entity_properties} AS e
            LEFT JOIN {xc_sql_metadata} AS m ON (e.metadata_id = m.metadata_id)
            WHERE e.metadata_id = %d';
  }
  $entity = db_fetch_object(db_query($sql, $metadata_id));
  $metadata = unserialize($entity->metadata);
  if ($metadata == FALSE) {
    xc_log_error('solr', 'Wrong serialization #' . $metadata_id . ': ' . $entity->metadata);
  }
  $entity->metadata = $metadata;
  $t2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/01 xc_solr_get_metadata'] += ($t2 - $t);

  //drupal_set_message(var_export($entity, TRUE));
  return $entity; // $source_id,
}

function xc_solr_get_metadata_by_identifier($identifier, $identifier_prefix) {
  $identifier_int = xc_util_get_identifier_int($identifier, $identifier_prefix);
  return xc_solr_get_metadata_by_identifier_int($identifier_int);
}

function xc_solr_get_metadata_by_identifier_int($identifier_int) {
  global $_oaiharvester_statistics, $_xc_solr_false_metadata;
  static $queries;

  $t = microtime(TRUE);
  if (empty($identifier_int)) {
    $_xc_solr_false_metadata['empty']++;
    return FALSE;
  }
  // xc_log_info('solr', 'get #' . $metadata_id);
  $ts1 = microtime(TRUE);
  if (!isset($queries)) {
    $queries = (object)array(
      'strategy' => 'join', // or it may be 'distinct'

      // SQL for JOIN
      'join'     => 'SELECT e.identifier, e.identifier_int, e.node_id, e.node_type, e.metadata_id, e.metadata_type, e.source_id,
                     m.metadata AS metadata
                     FROM {xc_entity_properties} AS e
                     LEFT JOIN {xc_sql_metadata} AS m ON (e.metadata_id = m.metadata_id)
                     WHERE e.identifier_int = %d',

      // SQL for getting entity properties
      'entity'   => 'SELECT identifier, identifier_int, node_id, node_type, metadata_id, metadata_type, source_id
                     FROM {xc_entity_properties}
                     WHERE identifier_int = %d',

      // SQL for getting metadata
      'metadata' => 'SELECT metadata FROM {xc_sql_metadata} WHERE metadata_id = %d'
    );
  }

  if ($queries->strategy == 'join') {
    $entity = db_fetch_object(db_query($queries->join, $identifier_int));
    $ts2 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier/sql'] += ($ts2 - $ts1);
    if ($entity != FALSE) {
      $tu1 = microtime(TRUE);
      $metadata = unserialize($entity->metadata);
      $tu2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier/unserialize'] += ($tu2 - $tu1);
      if ($metadata == FALSE) {
        $_xc_solr_false_metadata['metadata']++;
        xc_log_error('solr', 'Wrong serialization #' . $identifier . ' ('. $identifier_int . '): ' . $entity->metadata);
      }
      else {
        $_xc_solr_false_metadata['success']++;
      }
      $entity->metadata = $metadata;
    }
    else {
      $_xc_solr_false_metadata['entity']++;
    }
  }
  elseif ($queries->strategy == 2) { // 'distinct'
    $entity = db_fetch_object(db_query($queries->entity, $identifier_int));
    $ts2 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier/01 get entity'] += ($ts2 - $ts1);

    if ($entity != FALSE) {
      $entity->metadata = db_result(db_query($queries->metadata, $entity->metadata_id));
      $ts3 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier/02 get metadata'] += ($ts3 - $ts2);

      $tu1 = microtime(TRUE);
      $metadata = unserialize($entity->metadata);
      $tu2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier/03 unserialize'] += ($tu2 - $tu1);
      if ($metadata == FALSE) {
        $_xc_solr_false_metadata['metadata']++;
        xc_log_error('solr', 'Wrong serialization #' . $identifier . ' ('. $identifier_int . '): ' . $entity->metadata);
      }
      else {
        $_xc_solr_false_metadata['success']++;
      }
      $entity->metadata = $metadata;
    }
    else {
      $_xc_solr_false_metadata['entity']++;
    }
  }
  $t2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent/xc_solr_get_metadata_by_identifier'] += ($t2 - $t);

  return $entity; // $source_id,
}

function xc_solr_get_parent($entity) {
  global $_oaiharvester_statistics;
  $t_start = microtime(TRUE);
  $metadatas = array();

  // $parents = xc_entity_get_relationships($entity, 'parent', TRUE, FALSE, TRUE);
  $parents = xc_entity_get_parent_identifiers($entity->identifier_int);
  if ($parents) {
    foreach ($parents as $identifier_int) {
      // $metadata = xc_solr_get_metadata_by_identifier_int($identifier, $identifier_prefix);
      $metadata = xc_solr_get_metadata_by_identifier_int($identifier_int);
      if (empty($metadata)) {
        xc_log_error('solr', 'No metadata for ' . $identifier_int . ' (parent of an ' . $entity->metadata_type . ')');
      }
      else {
        $metadatas[] = $metadata;
      }
    }
  }
  else {
    if ($entity->metadata_type == 'manifestation') {
      $parents = $entity->metadata['xc:expressionManifested'];
    }
    elseif ($entity->metadata_type == 'expression') {
      $parents = $entity->metadata['xc:workExpressed'];
    }
    elseif ($entity->metadata_type == 'holdings') {
      $parents = $entity->metadata['xc:manifestationHeld'];
    }
    elseif ($entity->metadata_type == 'item') {
      $parents = $entity->metadata['xc:holdingsExemplified'];
    }
    if (!empty($parents)) {
      $identifier_prefix = _source_id_to_identifier_prefix($entity->source_id);
      foreach ($parents as $identifier) {
        // $metadata_id = _identifier_to_metadata_id($identifier['#value']);
        $metadata = xc_solr_get_metadata_by_identifier($identifier['#value'], $identifier_prefix);
        $t2 = microtime(TRUE);
        if (empty($metadata)) {
          xc_log_error('solr', 'No metadata for ' . $identifier['#value'] . ' (' . $entity->metadata_type . ')');
        }
        else {
          $metadatas[] = $metadata;
        }
      }
    }
  }
  $t_end = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/xc_solr_get_parent'] += ($t_end - $t_start);
  return $metadatas;
}

/**
 *
 * @param $type
 * @return unknown_type
 */
function xc_solr_metadata_to_solr_factory(&$object, $type, &$document = NULL) {
  switch ($type) {
    case 'iterate over fields':
      return xc_solr_metadata_to_solr_iterate_fields($object, $document); break;
    case 'iterate over indexes':
      return xc_solr_metadata_to_solr_iterate_indexes($object, $document); break;
  }
}

/**
 * Converts a metadata object to Solr document
 *
 * @param $object (Object)
 *   A metadata object.
 *   an example:
 *   identifier => 'oai:URMST:Transformation Service/10002'
 *   (optional) 'node_id' => ...
 *   (optional) 'node_type' => ...
 *   metadata_id => ...,
 *   metadata_type => ...,
 *   source_id => ...
 *   metadata => array(
 *     "@type" => "expression",
 *     "@id"   => "oai:mst.rochester.edu:MST/MARCToXCTransformation/14999",
 *     "xc:titleOfExpression" = array(array("#value" => "Philosophy of art history.")),
 *     "dcterms:type" => array(array("@type" => "dcterms:DCMIType", "#value" => "Text")),
 *     ...
 *   );
 */
function xc_solr_metadata_to_solr_iterate_fields(&$object, &$document) {
  global $_oaiharvester_statistics;
  static $cache = array(
    'unique_fieldnames' => array('serialized_tsn'),
    'subject_topic_facet_field' => array(
      array('#name' => 'subject_fc', '#type' => 'facet'),
      array('#name' => 'topic_fc', '#type' => 'facet'),
      array('#name' => 'subject_topic_fc', '#type' => 'facet'),
  ));

  $t_new = microtime(TRUE);
  if (is_null($document)) {
    $document = new Apache_Solr_DocumentXc();
  }
  $document->setUniqueFieldnames($cache['unique_fieldnames']);
  $t_new2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/new'] += ($t_new2 - $t_new);

  $debug = FALSE;
  //$debug = ($object->identifier == 'oai:URMST:Transformation Service/10002') ? TRUE : FALSE;

  $document->addField('id',              $object->identifier, FALSE, TRUE);
  if (isset($object->node_id)) {
    $document->addField('node_id_i',     $object->node_id, FALSE, TRUE);
  }
  if (isset($object->node_type)) {
    $document->addField('node_type_s',   $object->node_type, FALSE, TRUE);
  }
  $document->addField('metadata_id_s',   $object->metadata_id, FALSE, TRUE);
  $document->addField('metadata_type_s', $object->metadata_type, FALSE, TRUE);
  $document->addField('source_id_s',     $object->source_id, FALSE, TRUE);
  $document->addField('serialized_tsn',  serialize($object), FALSE, TRUE);
  $t_new3 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/common fields'] += ($t_new3 - $t_new2);
  $content = '';
  foreach ($object->metadata as $field => $values) {
    if (!isset($values) || empty($values)) {
      continue;
    }

    $solr_fields = array();
    unset($solr_field);

    // set field name
    if ($field == '@type') {
      $solr_field = 'type';
    }
    elseif ($field == '@id' || $field == 'metadata_id' || $field == 'xc_id') {
      $solr_field = 'id';
    }
    else {
      // fieldnames
      $t = microtime(TRUE);
      foreach (xc_solr_get_type_suffixes($field) as $suffix) {
        if ($suffix == '_dt') {
          $solr_fields[] = array(
            '#name' => xc_solr_get_solr_field_name($field, $suffix),
            '#type' => 'date_field'
          );
        }
        else {
          $solr_fields[] = xc_solr_get_solr_field_name($field, $suffix);
        }
      }
      $t2 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/solr fields'] += ($t2 - $t);
    }
    // delayed insert of type and id
    if (isset($solr_field)) {
      $solr_fields[] = array('#name' => $solr_field, '#type' => 'field');
    }

    // handle facet aliases
    $t = microtime(TRUE);
    $facets = xc_solr_get_facet_aliases($field);
    $t2 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/xc_solr_get_facet_aliases'] += ($t2 - $t);
    $solr_fields = array_merge($solr_fields, $facets);

    // handle values
    if (!is_array($values)) {
      if ($solr_field == 'type') {
        $document->addField($solr_field, $values, FALSE, TRUE);
      }
      elseif ($solr_field == 'id') {
        if (!$document->getField('id')) {
          $document->addField($solr_field, $values, FALSE, TRUE);
        }
      }
      else {
        drupal_set_message(t('not an array: %f=%v',
          array('%f' => $field, '%v' => var_export($values, TRUE))));
      }
    }
    // handle values in an array
    else {
      foreach ($values as $element_instance) {
        // problem in metadata format
        if (!is_array($element_instance)) {
          xc_log_error('Solr', 'not an array: ' . var_export($element_instance, TRUE));
        }

        $actual_solr_fields = $solr_fields;
        foreach ($element_instance as $key => $value) {
          // field value
          if ($key == '#value') {
            $content .= $value . ' ';
            if (!empty($facets)) {
              $t = microtime(TRUE);
              $cleaned_value = _xc_solr_clean($value);
              $t2 = microtime(TRUE);
              $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/_xc_solr_clean'] += ($t2 - $t);
            }
            foreach ($actual_solr_fields as $solr_field) {
              if ($solr_field['#type'] == 'facet' && $cleaned_value != '') {
                _xc_solr_add_field($document, $solr_field, $cleaned_value);
              }
              elseif ($value != '') {
                _xc_solr_add_field($document, $solr_field, $value);
              }
            }
          }
          // handling attributes
          else {
            // remove subject’s LCC, DDC
            if ($field == 'dcterms:subject' && $key = '@type'
                && in_array($value, array('dcterms:LCC', 'dcterms:DDC'))) {
              // remove subject facet
              $t = microtime(TRUE);
              xc_util_remove_array_element($actual_solr_fields, $cache['subject_topic_facet_field']);
              $t2 = microtime(TRUE);
              $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/xc_util_remove_array_element'] += ($t2 - $t);
            }
            $t = microtime(TRUE);
            $result = xc_solr_process_field_attribute($field, $key, $value, $debug);
            $t2 = microtime(TRUE);
            $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/xc_solr_process_field_attribute'] += ($t2 - $t);
            if ($result) {
              if (count($result['as_key_exclusively']) > 0) {
                $actual_solr_fields = $result['as_key_exclusively'];
              }
              else {
                $actual_solr_fields = array_merge($actual_solr_fields, $result['value_as_key']);
              }
              if (!empty($result['value_as_value'])) {
                foreach ($result['value_as_value'] as $attr_field) {
                  if ($attr_field['#type'] == 'facet' && $cleaned_value != '') {
                    _xc_solr_add_field($document, $attr_field, $cleaned_value);
                  }
                  elseif ($value != '') {
                    _xc_solr_add_field($document, $attr_field, $value);
                  }
                }
              }
            }
          } // handling attributes
        } // foreach element_instances
      } // foreach values
    } // processing arrays
  } // processing metadata
  $field = 'text2';
  _xc_solr_add_field($document, $field, $content);
  return $document;
}

/**
 * Converts a metadata object to Solr document
 *
 * @param $object (Object)
 *   A metadata object.
 *   an example:
 *   identifier => 'oai:URMST:Transformation Service/10002'
 *   (optional) 'node_id' => ...
 *   (optional) 'node_type' => ...
 *   metadata_id => ...,
 *   metadata_type => ...,
 *   source_id => ...
 *   metadata => array(
 *     "@type" => "expression",
 *     "@id"   => "oai:mst.rochester.edu:MST/MARCToXCTransformation/14999",
 *     "xc:titleOfExpression" = array(array("#value" => "Philosophy of art history.")),
 *     "dcterms:type" => array(array("@type" => "dcterms:DCMIType", "#value" => "Text")),
 *     ...
 *   );
 */
function xc_solr_metadata_to_solr_iterate_indexes(&$object, &$document = NULL) {
  global $_oaiharvester_statistics, $_xc_solr_conditional_facets;
  static $indexes;

  //xc_log_info('solr', var_export($object, TRUE));
  static $cache = array(
    'unique_fieldnames' => array('serialized_tsn'),
    'subject_topic_facet_field' => array(
      array('#name' => 'subject_fc', '#type' => 'facet'),
      array('#name' => 'topic_fc', '#type' => 'facet'),
      array('#name' => 'subject_topic_fc', '#type' => 'facet'),
  ));

  $t_new = microtime(TRUE);
  if (is_null($document)) {
    $new_document = TRUE;
    $document = new Apache_Solr_DocumentXc();
  }
  else {
    $new_document = FALSE;
  }
  $document->setUniqueFieldnames($cache['unique_fieldnames']);
  $t_new2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/new'] += ($t_new2 - $t_new);

  $debug = FALSE;
  //$debug = ($object->identifier == 'oai:URMST:Transformation Service/10002') ? TRUE : FALSE;
  if ($new_document) { // $object->metadata_type == 'manifestation') {
    $document->addField('id',              $object->identifier, FALSE, TRUE);

    if (isset($object->node_id)) {
      $document->addField('node_id_i',     $object->node_id, FALSE, TRUE);
    }

    if (isset($object->node_type)) {
      $document->addField('node_type_s',   $object->node_type, FALSE, TRUE);
    }

    $document->addField('metadata_id_s',   $object->metadata_id, FALSE, TRUE);
    $document->addField('metadata_type_s', $object->metadata_type, FALSE, TRUE);
    $document->addField('source_id_s',     $object->source_id, FALSE, TRUE);
    // $document->addField('serialized_tsn',  serialize($object), FALSE, TRUE);
    $document->addField('type',            $object->metadata_type, FALSE, TRUE); // $object->metadata['@type']
    // TODO: identifier_int
  }
  $t_new3 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/common fields'] += ($t_new3 - $t_new2);
  if (!isset($indexes)) {
    $indexes = xc_solr_fields2index_list();
  }
  foreach ($indexes as $field => $field_properties) {
    if (!isset($object->metadata[$field])) {
      continue;
    }
    /*
    if (isset($field_properties['value_as_key']) || isset($field_properties['value_as_value'])) {
      drupal_set_message($field . ': ' . var_export($field_properties, TRUE));
    }
    */

    // main field value
    $tm = microtime(TRUE);
    if (isset($field_properties['general_fields']) && isset($object->metadata[$field])) {
      $values = array();
      foreach ($object->metadata[$field] as $instance) {
        $values[] = $instance['#value'];
      }
      xc_solr_add_fields($document, $field_properties['general_fields'], $values);
    }

    // date fields
    if (isset($field_properties['date_fields']) && isset($object->metadata[$field])) {
      $values = array();
      foreach ($object->metadata[$field] as $instance) {
        $temp = xc_solr_to_date($instance['#value'], $field);
        if (strlen($temp) == 20) {
          $values[] = $temp;
        }
      }
      xc_solr_add_fields($document, $field_properties['date_fields'], $values);
    }

    // use attribute value as field value
    $tm2 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/main value'] += ($tm2 - $tm);
    if (isset($field_properties['value_as_value'])) {
      foreach ($field_properties['value_as_value'] as $attribute) {
        if (isset($object->metadata[$field][$attribute])) {
          xc_solr_add_fields($document, $field_properties['value_as_value'][$attribute], $object->metadata[$field][$attribute]);
          xc_solr_add_fields($document, $field_properties['general_fields'], $object->metadata[$field][$attribute]);
        }
      }
    }

    // use attribute value as part of field name
    $tm3 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/value_as_value'] += ($tm3 - $tm2);
    if (isset($field_properties['value_as_key'])) {
      foreach ($field_properties['value_as_key'] as $attribute => $attribute_properties) {
        for ($len = count($object->metadata[$field]), $i = 0; $i < $len; $i++) {
          if (isset($object->metadata[$field][$i]['@' . $attribute])) {
            $key = $object->metadata[$field][$i]['@' . $attribute];
            if (isset($attribute_properties['values'][$key])) {
              xc_solr_add_fields($document, $attribute_properties['values'][$key], $object->metadata[$field][$i]['#value']);
            }
          }
        }
      }
    }

    // use attribute value as part of field name and do not use main field
    $tm4 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/value_as_value'] += ($tm4 - $tm3);
    // TODO: revision
    if (isset($field_properties['as_key_exclusively'])) {
      foreach ($field_properties['as_key_exclusively'] as $attribute) {
        $values = array();
        foreach ($object->metadata[$field] as $instance) {
          if (isset($instance[$attribute])) {
            $values[] = $instance[$attribute];
          }
        }
        if (!empty($values)) {
          xc_solr_add_fields($document, $field_properties['as value'][$attribute], $values);
        }
      }
    }

    // date facet
    $tm5 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/as_key_exclusively'] += ($tm5 - $tm4);
    if (isset($field_properties['date_facets'])) {
      $values = array();
      foreach ($object->metadata[$field] as $instance) {
        $temp = xc_solr_to_date($instance['#value'], $field);
        if (strlen($temp) == 20) {
          $values[] = $temp;
        }
      }
      xc_solr_add_fields($document, $field_properties['date_facets'], $values);
    }

    // facet
    $tm6 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/date_facet'] += ($tm6 - $tm5);
    if (isset($field_properties['facets'])) {
      $values = array();
      $tm61 = microtime(TRUE);
      foreach ($object->metadata[$field] as $instance) {
        $value = _xc_solr_clean($instance['#value']);
        $values[] = $value;
      }
      $tm62 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/facet/get values'] += ($tm62 - $tm61);
      xc_solr_add_fields($document, $field_properties['facets'], $values);
      $tm63 = microtime(TRUE);
      $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/facet/add fields'] += ($tm63 - $tm62);
    }
    $tm7 = microtime(TRUE);
    $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/facet'] += ($tm7 - $tm6);

    // collect conditional facets. We will run all this at the end of processing
    if (isset($field_properties['conditional_facets'])) {
      foreach ($field_properties['conditional_facets'] as $facet) {
        if (!isset($_xc_solr_conditional_facets[$facet])) {
          $_xc_solr_conditional_facets[$facet] = array();
        }
        if (!isset($_xc_solr_conditional_facets[$facet][$field])) {
          $_xc_solr_conditional_facets[$facet][$field] = 1;
        }
      }
    }
  }

  // text field, which contains everything
  $tm1 = microtime(TRUE);
  $content = '';
  if (is_array($object->metadata)) {
    foreach ($object->metadata as $field => $values) {
      if (!isset($values) || empty($values) || !is_array($values)) {
        continue;
      }
      foreach ($values as $element_instance) {
        if (is_array($element_instance)) {
          foreach ($element_instance as $key => $value) {
            if ($key == '#value') {
              $content .= $value . ' ';
            } // foreach element_instances
          }
        } // foreach values
        else { // '@id', '@type'
          $content .= $element_instance . ' ';
        }
      } // processing arrays
    }
  } // processing metadata
  $document->addField('text', $content, FALSE, TRUE);
  $t2 = microtime(TRUE);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr/text'] += ($t2 - $tm1);
  $_oaiharvester_statistics['03 step3/02 iterator/01 xc_solr_onestop_indexer/toSolr'] += ($t2 - $t_new);
  return $document;
}

function xc_solr_index_general_value_as_key($metadata, $field_name, $field_properties, $document = NULL) {
  if (is_null($document)) {
    $values = array();
  }
  foreach ($field_properties['value_as_key'] as $attribute => $attribute_properties) {
    for ($len = count($metadata[$field_name]), $i = 0; $i < $len; $i++) {
      if (isset($metadata[$field_name][$i]['@' . $attribute])) {
        $key = $metadata[$field_name][$i]['@' . $attribute];
        if (isset($attribute_properties['values'][$key])) {
          if (!is_null($document)) {
            xc_solr_add_fields($document, $attribute_properties['values'][$key], $metadata[$field_name][$i]['#value']);
          }
          else {
            $values[] = $metadata[$field_name][$i]['#value'];
          }
        }
      }
    }
  }
  if (is_null($document)) {
    return $values;
  }
}

/**
 * Adds field to Solr Document
 *
 * @param $document (Object)
 *   Reference to Solr Document
 * @param $fields (array)
 *   The list of fields
 * @param $values (array)
 *   The list of values
 * @param $function (function)
 *   An optional function which will be applied to all values
 */
function xc_solr_add_fields(&$document, $fields, $values, $function = NULL) {

  // make it array
  if (!is_array($values)) {
    $values = (array)$values;
  }

  if (!is_array($fields)) {
    $fields = (array)$fields;
  }

  // run function if there is
  if (!is_null($function)) {
    array_walk($values, $function);
  }

  foreach ($fields as $field) {
    // check whether the field values has to be fieltered
    if (xc_solr_is_field_with_excludable_values($field)) {
      $document->addField($field, xc_solr_exclude_values($field, $values), FALSE, TRUE);
    }
    else {
      $document->addField($field, $values, FALSE, TRUE);
    }
  }
}

/**
 * Create a cache directory for the Solr input (XML or JSON) files
 *
 * @return (String)
 *   The cache directory's name
 */
function xc_solr_create_input_cache() {

  $cache_dir = file_directory_path() . '/xc_solr_input_cache/';
  if (!file_check_directory($cache_dir, FILE_CREATE_DIRECTORY)) {
    xc_log_error('harvester', t('The Drupal default "file" directory is not writable. Check out your Drupal settings, or turn off the caching of harvested responses. Your current file directory is %file.', array('%file' => $file_dir)));
    $cache_dir = '';
  }

  return $cache_dir;
}

/**
 * Remove all JSON files from the cache directory
 *
 * @param $dir (String)
 *   The name of directory
 */
function xc_solr_purge_input_cache($dir) {
  $mask = $dir . '/upload.*.json';
  array_map('unlink', glob($mask));
}

/**
 * Updates the context message in Batch API
 *
 * @param $context (Array)
 *   Reference to the Batch API context array
 * @param $preparation_time (int)
 *   The time the preparation process takes (in ms)
 */
function xc_solr_onestop_indexer_update_context_message(&$context, $preparation_time) {

  $percent       = ($context['sandbox']['offset'] / $context['sandbox']['max']);
  $now = time();
  $time_elapsed  = ((int)$now - (int)$context['sandbox']['start_time']);
  $time_iteration = $time_elapsed - $preparation_time;
  $time_remained = ((int)($time_iteration / $percent) - $time_iteration);
  $time_total    = $time_elapsed + $time_remained;
  $context['message'] = t('%offset / %max. Elapsed time: %time_elapsed, remained appr.: %time_remained, total appr.: %time_total.<br/>',
    array(
      '%offset' => $context['sandbox']['offset'],
      '%max' => $context['sandbox']['max'],
      '%time_elapsed' =>  oaiharvester_sec_to_time($time_elapsed),
      '%time_remained' => oaiharvester_sec_to_time($time_remained),
      '%time_total' => oaiharvester_sec_to_time($time_total),
    )
  );
  $context['message'] .= t('Warning! Do not close this window untill reindex is complete!<br/>');
  $current_batch = batch_get();
  $link = l(t('Stop harvesting'), 'batch', array('query' => array('op' => 'finished', 'id' => $current_batch['id'])));
  $context['message'] .= ' ' . $link . ' ' . t('(You will be able to restart or start over.)');
}

/**
 * Sets the Solr dump file name
 *
 * It manipulates the context array, and sets $context['sandbox']['upload']['file'].
 *
 * @param $context (Array)
 *   Reference to the Batch API's $context array.
 */
function xc_solr_create_input_cache_file_name(&$context) {
  $ext = $context['sandbox']['asJSON'] ? 'json' : 'xml';
  $context['sandbox']['upload']['file'] = sprintf('%s/upload.%06d.%s',
    $context['sandbox']['upload']['cache_dir'],
    $context['sandbox']['upload']['file_counter'],
    $ext
  );
  return $context['sandbox']['upload']['file'];
}

/**
 * Get the actual Solr strategy for indexing sortable fields.
 *
 * @return (int)
 *   The constans defined for the appropriate strategy. Possible values:
 *   - XC_SOLR_SORT_STRATEGY_MINMAX: for creating two fields for minimum and maximum values
 *   - XC_SOLR_SORT_STRATEGY_FIRST: for creating field from the first value
 */
function xc_solr_get_sortstrategy() {
  return XC_SOLR_SORT_STRATEGY_MINMAX;
}

/**
 * Returns the actual metadata process strategy, actually XC_SOLR_METADATA_PROCESS_STRATEGY_MERGE_FIRST
 *
 * @return int
 *   The actual strategy as constant
 */
function xc_solr_get_metadata_process_strategy() {
  return XC_SOLR_METADATA_PROCESS_STRATEGY_MERGE_FIRST; // the other option is XC_SOLR_METADATA_PROCESS_STRATEGY_INDEX_FIRST
}

/**
 * Get those fields, which are in multiple entities, but we need them separated
 *
 * <code>
 * array(
      'manifestation' => array('xc:recordID' => 'bibid_i'),
      'holdings' => array('xc:recordID' => 'holdingsid_i'),
    );
   </code>

 * @return (Array)
 *   schema field, Solr field pairs grouped by entities
 */
function xc_solr_get_conflicting_fields() {
  static $cache;

  if (!isset($cache)) {
    $cache = array(
      'manifestation' => array('xc:recordID' => 'bibid_i'),
      'holdings' => array('xc:recordID' => 'holdingsid_i'),
    );
  }

  return $cache;
}

/**
 * Get the list of excludable values.
 *
 * @return (Array)
 *   An associative array in which the keys are the Solr field names, and the values
 *   are the array of excludable values.
 */
function xc_solr_get_excludable_values() {
  $values = array(
    'subject_fc' => array('Text'),
  );
  return $values;
}

/**
 * Checks whether the field has excludable values
 *
 * @param $field (String)
 *   The name of the field to check
 *
 * @return (Boolean)
 *   TRUE if the field has excludable value, FALSE otherwise
 */
function xc_solr_is_field_with_excludable_values($field) {
  static $cache, $list_of_fields;

  if (!isset($list_of_fields)) {
    $list_of_fields = array_keys(xc_solr_get_excludable_values());
  }

  if (!isset($cache[$field])) {
    $cache[$field] = in_array($field, $list_of_fields);
  }

  return $cache[$field];
}

/**
 * Filters out the excludable values among a field's values
 *
 * @param $field (String)
 *   The name of Solr field
 * @param $values (Array)
 *   The field's original values
 *
 * @return (Array)
 *   The fields filtered values
 */
function xc_solr_exclude_values($field, $values) {
  static $settings;

  if (!isset($settings)) {
    $settings = xc_solr_get_excludable_values();
  }

  return array_diff($values, $settings[$field]);
}
